@@ -29,6 +29,8 @@
 #include "matrix.h"
 #include "power.h"
 #include "lst.h"
+#include "function.h"
+#include "symbol.h"
 #include "relational.h"
 #include "utils.h"
 
@@ -126,7 +128,7 @@ ex ex::subs(const lst & ls, const lst & lr, unsigned options) const
 
 	// Convert the lists to a map
 	exmap m;
-	for (lst::const_iterator its = ls.begin(), itr = lr.begin(); its != ls.end(); ++its, ++itr) {
+	for (auto its = ls.begin(), itr = lr.begin(); its != ls.end(); ++its, ++itr) {
 		m.insert(std::make_pair(*its, *itr));
 
 		// Search for products and powers in the expressions to be substituted
@@ -165,8 +167,8 @@ ex ex::subs(const ex & e, unsigned options) const
 		// Argument is a list: convert it to a map
 		exmap m;
 		GINAC_ASSERT(is_a<lst>(e));
-		for (lst::const_iterator it = ex_to<lst>(e).begin(); it != ex_to<lst>(e).end(); ++it) {
-			ex r = *it;
+		for (auto r : ex_to<lst>(e)) {
+			
 			if (!r.info(info_flags::relation_equal))
 				throw(std::invalid_argument("basic::subs(ex): argument must be a list of equations"));
 			const ex & s = r.op(0);
@@ -225,10 +227,58 @@ ex & ex::operator[](size_t i)
 	return (*bp)[i];
 }
 
+bool ex::is_equal(const ex & other) const
+{
+#ifdef GINAC_COMPARE_STATISTICS
+	compare_statistics.total_is_equals++;
+#endif
+	if (bp == other.bp)  // trivial case: both expressions point to same basic
+		return true;
+#ifdef GINAC_COMPARE_STATISTICS
+	compare_statistics.nontrivial_is_equals++;
+#endif
+	if (is_exactly_a<numeric>(*this) and is_exactly_a<numeric>(other))
+		return ex_to<numeric>(*this).is_equal(ex_to<numeric>(other));
+	const bool equal = bp->is_equal(*other.bp);
+#if 0
+	if (equal) {
+		// Expressions point to different, but equal, trees: conserve
+		// memory and make subsequent compare() operations faster by
+		// making both expressions point to the same tree.
+		share(other);
+	}
+#endif
+	return equal;
+}
+
+int ex::compare(const ex & other) const
+{
+#ifdef GINAC_COMPARE_STATISTICS
+	compare_statistics.total_compares++;
+#endif
+	if (bp == other.bp)  // trivial case: both expressions point to same basic
+		return 0;
+#ifdef GINAC_COMPARE_STATISTICS
+	compare_statistics.nontrivial_compares++;
+#endif
+	if (is_exactly_a<numeric>(*this) and is_exactly_a<numeric>(other))
+		return ex_to<numeric>(*this).compare_same_type(ex_to<numeric>(other));
+	const int cmpval = bp->compare(*other.bp);
+#if 1
+	if (cmpval == 0) {
+		// Expressions point to different, but equal, trees: conserve
+		// memory and make subsequent compare() operations faster by
+		// making both expressions point to the same tree.
+		share(other);
+	}
+#endif
+	return cmpval;
+}
+
 /** Left hand side of relational expression. */
 ex ex::lhs() const
 {
-	if (!is_a<relational>(*this))
+	if (!is_exactly_a<relational>(*this))
 		throw std::runtime_error("ex::lhs(): not a relation");
 	return bp->op(0);
 }
@@ -236,7 +286,7 @@ ex ex::lhs() const
 /** Right hand side of relational expression. */
 ex ex::rhs() const
 {
-	if (!is_a<relational>(*this))
+	if (!is_exactly_a<relational>(*this))
 		throw std::runtime_error("ex::rhs(): not a relation");
 	return bp->op(1);
 }
@@ -244,10 +294,10 @@ ex ex::rhs() const
 /** Check whether expression is a polynomial. */
 bool ex::is_polynomial(const ex & vars) const
 {
-	if (is_a<lst>(vars)) {
+	if (is_exactly_a<lst>(vars)) {
 		const lst & varlst = ex_to<lst>(vars);
-		for (lst::const_iterator i=varlst.begin(); i!=varlst.end(); ++i)
-			if (!bp->is_polynomial(*i))
+		for (const auto & elem : varlst)
+			if (!bp->is_polynomial(elem))
 				return false;
 		return true;
 	}
@@ -255,6 +305,15 @@ bool ex::is_polynomial(const ex & vars) const
 		return bp->is_polynomial(vars);
 }
 
+bool ex::is_zero() const {
+#ifndef _MSC_VER
+	  extern const ex _ex0;
+#endif
+        if (!is_exactly_a<numeric>(*this))
+                return is_equal(_ex0);
+        return ex_to<numeric>(*this).is_zero();
+}
+
 /** Check whether expression is zero or zero matrix. */
 bool ex::is_zero_matrix() const
 {
@@ -262,7 +321,7 @@ bool ex::is_zero_matrix() const
 		return  true;
 	else {
 		ex e = evalm();
-		return is_a<matrix>(e) && ex_to<matrix>(e).is_zero_matrix();
+		return is_exactly_a<matrix>(e) && ex_to<matrix>(e).is_zero_matrix();
 	}
 }
 
@@ -270,8 +329,8 @@ bool ex::is_integer_one() const
 {
     if (!is_exactly_a<numeric>(*this))
         return false;
-    numeric num = ex_to<numeric>(*this);
-    return num.is_integer() && num.is_equal(*_num1_p);
+    return ex_to<numeric>(*this).is_integer()
+        and ex_to<numeric>(*this).is_equal(*_num1_p);
 }
   
 bool ex::is_integer_pmone() const
@@ -279,14 +338,27 @@ bool ex::is_integer_pmone() const
     if (!is_exactly_a<numeric>(*this))
         return false;
     numeric num = ex_to<numeric>(*this);
-    return ((num.is_integer()) &&
-            (num.is_equal(*_num1_p) || num.is_equal(*_num_1_p)));
+    return ((ex_to<numeric>(*this).is_integer())
+        and (ex_to<numeric>(*this).is_equal(*_num1_p)
+            or ex_to<numeric>(*this).is_equal(*_num_1_p)));
+}
+
+void ex::set_domain(unsigned d)
+{
+        if (is_exactly_a<symbol>(*this)) {
+                symbol &s = dynamic_cast<symbol&>(*bp);
+                s.set_domain(d);
+        }
+        else if (is_exactly_a<function>(*this)) {
+                function &f = dynamic_cast<function&>(*bp);
+                f.set_domain(d);
+        }
 }
 
 size_t ex::nsymbols() const
 {
 	int res = 0;
-	if (is_a<symbol>(*this)) {
+	if (is_exactly_a<symbol>(*this)) {
 		res=1;
 	} else {
 		for (size_t i=0; i < nops(); i++)
