@@ -70,7 +70,6 @@ class const_iterator;
 class const_preorder_iterator;
 class const_postorder_iterator;
 
-
 /** Lightweight wrapper for GiNaC's symbolic objects.  It holds a pointer to
  *  the other object in order to do garbage collection by the method of
  *  reference counting.  I.e., it is a smart pointer.  Also, the constructor
@@ -127,7 +126,7 @@ public:
 
 	// evaluation
 	ex eval(int level = 0) const { return bp->eval(level); }
-	ex evalf(int level = 0, PyObject* parent=NULL) const 
+	ex evalf(int level = 0, PyObject* parent=nullptr) const 
 	{ return bp->evalf(level, parent); }
 	ex evalm() const { return bp->evalm(); }
 	ex eval_ncmul(const exvector & v) const { return bp->eval_ncmul(v); }
@@ -221,15 +220,13 @@ public:
 	ex simplify_indexed(unsigned options = 0) const;
 	ex simplify_indexed(const scalar_products & sp, unsigned options = 0) const;
 
+	// domains
+	void set_domain(unsigned d);
+
 	// comparison
 	int compare(const ex & other) const;
 	bool is_equal(const ex & other) const;
-	bool is_zero() const { 
-#ifndef _MSC_VER
-	  extern const ex _ex0;
-#endif
-	  return is_equal(_ex0); 
-	}
+	bool is_zero() const;
 	bool is_zero_matrix() const;
         bool is_integer_one() const;
         bool is_integer_pmone() const;
@@ -246,7 +243,7 @@ public:
 	unsigned return_type() const { return bp->return_type(); }
 	tinfo_t return_type_tinfo() const { return bp->return_type_tinfo(); }
 
-	unsigned gethash() const { return bp->gethash(); }
+	long gethash() const { return bp->gethash(); }
 
 private:
 	static ptr<basic> construct_from_basic(const basic & other);
@@ -327,53 +324,6 @@ ex::ex(const std::string &s, const ex &l) : bp(construct_from_string_and_lst(s,
 	GINAC_ASSERT(bp->flags & status_flags::dynallocated);
 }
 
-inline
-int ex::compare(const ex & other) const
-{
-#ifdef GINAC_COMPARE_STATISTICS
-	compare_statistics.total_compares++;
-#endif
-	if (bp == other.bp)  // trivial case: both expressions point to same basic
-		return 0;
-#ifdef GINAC_COMPARE_STATISTICS
-	compare_statistics.nontrivial_compares++;
-#endif
-	const int cmpval = bp->compare(*other.bp);
-#if 1
-	if (cmpval == 0) {
-		// Expressions point to different, but equal, trees: conserve
-		// memory and make subsequent compare() operations faster by
-		// making both expressions point to the same tree.
-		share(other);
-	}
-#endif
-	return cmpval;
-}
-
-inline
-bool ex::is_equal(const ex & other) const
-{
-#ifdef GINAC_COMPARE_STATISTICS
-	compare_statistics.total_is_equals++;
-#endif
-	if (bp == other.bp)  // trivial case: both expressions point to same basic
-		return true;
-#ifdef GINAC_COMPARE_STATISTICS
-	compare_statistics.nontrivial_is_equals++;
-#endif
-	const bool equal = bp->is_equal(*other.bp);
-#if 0
-	if (equal) {
-		// Expressions point to different, but equal, trees: conserve
-		// memory and make subsequent compare() operations faster by
-		// making both expressions point to the same tree.
-		share(other);
-	}
-#endif
-	return equal;
-}
-
-
 // Iterators
 
 class const_iterator : public std::iterator<std::random_access_iterator_tag, ex, ptrdiff_t, const ex *, const ex &> {
@@ -385,7 +335,7 @@ public:
 	const_iterator() throw() {}
 
 private:
-	const_iterator(const ex &e_, size_t i_) throw() : e(e_), i(i_) {}
+	const_iterator(ex e_, size_t i_) throw() : e(std::move(e_)), i(i_) {}
 
 public:
 	// This should return an ex&, but that would be a reference to a
@@ -397,9 +347,9 @@ public:
 
 	// This should return an ex*, but that would be a pointer to a
 	// temporary value
-	std::auto_ptr<ex> operator->() const
+	std::unique_ptr<ex> operator->() const
 	{
-		return std::auto_ptr<ex>(new ex(operator*()));
+		return std::unique_ptr<ex>(new ex(operator*()));
 	}
 
 	ex operator[](difference_type n) const
@@ -503,7 +453,7 @@ protected:
 namespace internal {
 
 struct _iter_rep {
-	_iter_rep(const ex &e_, size_t i_, size_t i_end_) : e(e_), i(i_), i_end(i_end_) {}
+	_iter_rep(ex e_, size_t i_, size_t i_end_) : e(std::move(e_)), i(i_), i_end(i_end_) {}
 
 	bool operator==(const _iter_rep &other) const throw()
 	{
@@ -782,7 +732,7 @@ inline ex collect(const ex & thisex, const ex & s, bool distributed = false)
 inline ex eval(const ex & thisex, int level = 0)
 { return thisex.eval(level); }
 
-inline ex evalf(const ex & thisex, int level = 0, PyObject* parent=NULL)
+inline ex evalf(const ex & thisex, int level = 0, PyObject* parent=nullptr)
 { return thisex.evalf(level, parent); }
 
 inline ex evalm(const ex & thisex)
@@ -860,7 +810,7 @@ protected:
 	ex (*ptr)(const ex &);
 public:
 	explicit pointer_to_map_function(ex x(const ex &)) : ptr(x) {}
-	ex operator()(const ex & e) { return ptr(e); }
+	ex operator()(const ex & e) override { return ptr(e); }
 };
 
 template<class T1>
@@ -870,7 +820,7 @@ protected:
 	T1 arg1;
 public:
 	explicit pointer_to_map_function_1arg(ex x(const ex &, T1), T1 a1) : ptr(x), arg1(a1) {}
-	ex operator()(const ex & e) { return ptr(e, arg1); }
+	ex operator()(const ex & e) override { return ptr(e, arg1); }
 };
 
 template<class T1, class T2>
@@ -881,7 +831,7 @@ protected:
 	T2 arg2;
 public:
 	explicit pointer_to_map_function_2args(ex x(const ex &, T1, T2), T1 a1, T2 a2) : ptr(x), arg1(a1), arg2(a2) {}
-	ex operator()(const ex & e) { return ptr(e, arg1, arg2); }
+	ex operator()(const ex & e) override { return ptr(e, arg1, arg2); }
 };
 
 template<class T1, class T2, class T3>
@@ -893,7 +843,7 @@ protected:
 	T3 arg3;
 public:
 	explicit pointer_to_map_function_3args(ex x(const ex &, T1, T2, T3), T1 a1, T2 a2, T3 a3) : ptr(x), arg1(a1), arg2(a2), arg3(a3) {}
-	ex operator()(const ex & e) { return ptr(e, arg1, arg2, arg3); }
+	ex operator()(const ex & e) override { return ptr(e, arg1, arg2, arg3); }
 };
 
 template<class C>
@@ -903,7 +853,7 @@ protected:
 	C &c;
 public:
 	explicit pointer_to_member_to_map_function(ex (C::*member)(const ex &), C &obj) : ptr(member), c(obj) {}
-	ex operator()(const ex & e) { return (c.*ptr)(e); }
+	ex operator()(const ex & e) override { return (c.*ptr)(e); }
 };
 
 template<class C, class T1>
@@ -914,7 +864,7 @@ protected:
 	T1 arg1;
 public:
 	explicit pointer_to_member_to_map_function_1arg(ex (C::*member)(const ex &, T1), C &obj, T1 a1) : ptr(member), c(obj), arg1(a1) {}
-	ex operator()(const ex & e) { return (c.*ptr)(e, arg1); }
+	ex operator()(const ex & e) override { return (c.*ptr)(e, arg1); }
 };
 
 template<class C, class T1, class T2>
@@ -926,7 +876,7 @@ protected:
 	T2 arg2;
 public:
 	explicit pointer_to_member_to_map_function_2args(ex (C::*member)(const ex&, T1, T2), C &obj, T1 a1, T2 a2) : ptr(member), c(obj), arg1(a1), arg2(a2) {}
-	ex operator()(const ex & e) { return (c.*ptr)(e, arg1, arg2); }
+	ex operator()(const ex & e) override { return (c.*ptr)(e, arg1, arg2); }
 };
 
 template<class C, class T1, class T2, class T3>
@@ -939,7 +889,7 @@ protected:
 	T3 arg3;
 public:
 	explicit pointer_to_member_to_map_function_3args(ex (C::*member)(const ex &, T1, T2, T3), C &obj, T1 a1, T2 a2, T3 a3) : ptr(member), c(obj), arg1(a1), arg2(a2), arg3(a3) {}
-	ex operator()(const ex & e) { return (c.*ptr)(e, arg1, arg2, arg3); }
+	ex operator()(const ex & e) override { return (c.*ptr)(e, arg1, arg2, arg3); }
 };
 
 inline ex ex::map(ex f(const ex &)) const
