@@ -122,16 +122,16 @@ expairseq::expairseq(const exvector &v) : inherited(&expairseq::tinfo_static)
 	GINAC_ASSERT(is_canonical());
 }
 
-expairseq::expairseq(const epvector &v, const ex &oc, bool do_index_renaming)
-  : inherited(&expairseq::tinfo_static), overall_coeff(oc)
+expairseq::expairseq(const epvector &v, ex oc, bool do_index_renaming)
+  : inherited(&expairseq::tinfo_static), overall_coeff(std::move(oc))
 {
 	GINAC_ASSERT(is_a<numeric>(oc));
 	construct_from_epvector(v, do_index_renaming);
 	GINAC_ASSERT(is_canonical());
 }
 
-expairseq::expairseq(std::auto_ptr<epvector> vp, const ex &oc, bool do_index_renaming)
-  : inherited(&expairseq::tinfo_static), overall_coeff(oc)
+expairseq::expairseq(std::unique_ptr<epvector> vp, ex oc, bool do_index_renaming)
+  : inherited(&expairseq::tinfo_static), overall_coeff(std::move(oc))
 {
 	GINAC_ASSERT(vp.get()!=0);
 	GINAC_ASSERT(is_a<numeric>(oc));
@@ -148,17 +148,17 @@ expairseq::expairseq(const archive_node &n, lst &sym_lst) : inherited(n, sym_lst
 	, hashtabsize(0)
 #endif
 {
-	archive_node::archive_node_cit first = n.find_first("rest");
-	archive_node::archive_node_cit last = n.find_last("coeff");
+	auto first = n.find_first("rest");
+	auto last = n.find_last("coeff");
 	++last;
 	seq.reserve((last-first)/2);
 
-	for (archive_node::archive_node_cit loc = first; loc < last;) {
-		ex rest;
-		ex coeff;
-		n.find_ex_by_loc(loc++, rest, sym_lst);
-		n.find_ex_by_loc(loc++, coeff, sym_lst);
-		seq.push_back(expair(rest, coeff));
+	for (auto loc = first; loc < last;) {
+		ex lrest;
+		ex lcoeff;
+		n.find_ex_by_loc(loc++, lrest, sym_lst);
+		n.find_ex_by_loc(loc++, lcoeff, sym_lst);
+		seq.push_back(expair(lrest, lcoeff));
 	}
 
 	n.find_ex("overall_coeff", overall_coeff, sym_lst);
@@ -170,11 +170,9 @@ expairseq::expairseq(const archive_node &n, lst &sym_lst) : inherited(n, sym_lst
 void expairseq::archive(archive_node &n) const
 {
 	inherited::archive(n);
-	epvector::const_iterator i = seq.begin(), iend = seq.end();
-	while (i != iend) {
-		n.add_ex("rest", i->rest);
-		n.add_ex("coeff", i->coeff);
-		++i;
+	for (const auto & elem : seq) {
+		n.add_ex("rest", elem.rest);
+		n.add_ex("coeff", elem.coeff);
 	}
 	n.add_ex("overall_coeff", overall_coeff);
 }
@@ -278,8 +276,8 @@ bool expairseq::info(unsigned inf) const
 				return true;
 			else if (flags & status_flags::has_no_indices)
 				return false;
-			for (epvector::const_iterator i = seq.begin(); i != seq.end(); ++i) {
-				if (i->rest.info(info_flags::has_indices)) {
+			for (const auto & elem : seq) {
+				if (elem.rest.info(info_flags::has_indices)) {
 					this->setflag(status_flags::has_indices);
 					this->clearflag(status_flags::has_no_indices);
 					return true;
@@ -320,24 +318,21 @@ ex expairseq::stable_op(size_t i) const
 
 ex expairseq::map(map_function &f) const
 {
-	std::auto_ptr<epvector> v(new epvector);
+	std::unique_ptr<epvector> v(new epvector);
 	v->reserve(seq.size()+1);
 
-	epvector::const_iterator cit = seq.begin(), last = seq.end();
-	while (cit != last) {
-		v->push_back(split_ex_to_pair(f(recombine_pair_to_ex(*cit))));
-		++cit;
-	}
+	for (const auto & elem : seq)
+		v->push_back(split_ex_to_pair(f(recombine_pair_to_ex(elem))));
 
 	if (overall_coeff_equals_default())
-		return thisexpairseq(v, default_overall_coeff(), true);
+		return thisexpairseq(std::move(v), default_overall_coeff(), true);
 	else {
 		ex newcoeff = f(overall_coeff);
-		if(is_a<numeric>(newcoeff))
-			return thisexpairseq(v, newcoeff, true);
+		if(is_exactly_a<numeric>(newcoeff))
+			return thisexpairseq(std::move(v), newcoeff, true);
 		else {
 			v->push_back(split_ex_to_pair(newcoeff));
-			return thisexpairseq(v, default_overall_coeff(), true);
+			return thisexpairseq(std::move(v), default_overall_coeff(), true);
 		}
 	}
 }
@@ -348,30 +343,32 @@ ex expairseq::eval(int level) const
 	if ((level==1) && (flags &status_flags::evaluated))
 		return *this;
 	
-	std::auto_ptr<epvector> vp = evalchildren(level);
-	if (vp.get() == 0)
+	std::unique_ptr<epvector> vp = evalchildren(level);
+	if (vp.get() == nullptr)
 		return this->hold();
 	
-	return (new expairseq(vp, overall_coeff))->setflag(status_flags::dynallocated | status_flags::evaluated);
+	return (new expairseq(std::move(vp), overall_coeff))->setflag(status_flags::dynallocated | status_flags::evaluated);
 }
 
-epvector* conjugateepvector(const epvector&epv)
+epvector* conjugateepvector(const epvector& epv)
 {
-	epvector *newepv = 0;
-	for (epvector::const_iterator i=epv.begin(); i!=epv.end(); ++i) {
-		if(newepv) {
-			newepv->push_back(i->conjugate());
+	epvector *newepv = nullptr;
+	for (const auto & elem : epv) {
+		if (newepv) {
+			newepv->push_back(elem.conjugate());
 			continue;
 		}
-		expair x = i->conjugate();
-		if (x.is_equal(*i)) {
+		expair x = elem.conjugate();
+		if (x.is_equal(elem)) {
 			continue;
 		}
 		newepv = new epvector;
 		newepv->reserve(epv.size());
-		for (epvector::const_iterator j=epv.begin(); j!=i; ++j) {
-			newepv->push_back(*j);
-		}
+                for (const auto & elem2 : epv)
+                        if (&elem2 != &elem)
+                                newepv->push_back(elem2);
+                        else
+                                break;
 		newepv->push_back(x);
 	}
 	return newepv;
@@ -379,13 +376,12 @@ epvector* conjugateepvector(const epvector&epv)
 
 ex expairseq::conjugate() const
 {
-	epvector* newepv = conjugateepvector(seq);
+	std::unique_ptr<epvector> newepv(conjugateepvector(seq));
 	ex x = overall_coeff.conjugate();
 	if (!newepv && are_ex_trivially_equal(x, overall_coeff)) {
 		return *this;
 	}
 	ex result = thisexpairseq(newepv ? *newepv : seq, x);
-	delete newepv;
 	return result;
 }
 
@@ -429,16 +425,16 @@ bool expairseq::match(const ex & pattern, lst & repl_lst) const
 			ex p = ex_to<expairseq>(pattern).stable_op(i);
 			if (has_global_wildcard && p.is_equal(global_wildcard))
 				continue;
-			exvector::iterator it = ops.begin(), itend = ops.end();
+			auto it = ops.begin(), itend = ops.end();
 			while (it != itend) {
-				lst::const_iterator last_el = repl_lst.end();
+				auto last_el = repl_lst.end();
 				--last_el;
 				if (it->match(p, tmp_repl)) {
 					ops.erase(it);
 					goto found;
 				}
 				while(true) {
-					lst::const_iterator next_el = last_el;
+					auto next_el = last_el;
 					++next_el;
 					if(next_el == repl_lst.end())
 						break;
@@ -457,21 +453,22 @@ found:		;
 			// it has already been matched before, in which case the matches
 			// must be equal)
 			size_t num = ops.size();
-			std::auto_ptr<epvector> vp(new epvector);
+			std::unique_ptr<epvector> vp(new epvector);
 			vp->reserve(num);
 			for (size_t i=0; i<num; i++)
 				vp->push_back(split_ex_to_pair(ops[i]));
-			ex rest = thisexpairseq(vp, default_overall_coeff());
-            for (lst::const_iterator it = tmp_repl.begin(); it != tmp_repl.end(); ++it) {
-                if (it->op(0).is_equal(global_wildcard)) {
-                    if (rest.is_equal(it->op(1))) {
+
+                        ex rest = thisexpairseq(std::move(vp), default_overall_coeff());
+                        for (const auto & elem : tmp_repl) {
+                                if (elem.op(0).is_equal(global_wildcard)) {
+                                    if (rest.is_equal(elem.op(1))) {
+                                        repl_lst = tmp_repl;
+                                        return true;
+                                        }
+                                return false;
+                                }
+                        }
                         repl_lst = tmp_repl;
-                        return true;
-                    }
-                return false;
-                }
-            }
-            repl_lst = tmp_repl;
 			repl_lst.append(global_wildcard == rest);
 			return true;
 
@@ -491,9 +488,9 @@ found:		;
 
 ex expairseq::subs(const exmap & m, unsigned options) const
 {
-	std::auto_ptr<epvector> vp = subschildren(m, options);
+	std::unique_ptr<epvector> vp = subschildren(m, options);
 	if (vp.get())
-		return ex_to<basic>(thisexpairseq(vp, overall_coeff, (options & subs_options::no_index_renaming) == 0));
+		return ex_to<basic>(thisexpairseq(std::move(vp), overall_coeff, (options & subs_options::no_index_renaming) == 0));
 	else if ((options & subs_options::algebraic) && is_exactly_a<mul>(*this))
 		return static_cast<const mul *>(this)->algebraic_subs_mul(m, options);
 	else
@@ -504,11 +501,8 @@ ex expairseq::subs(const exmap & m, unsigned options) const
 numeric expairseq::calc_total_degree() const
 {
 	numeric deg = 0;
-	epvector::const_iterator cit = seq.begin();
-	epvector::const_iterator last = seq.end();
-	for (; (cit!=last); ++cit) {
-		deg = deg.add(ex_to<numeric>(cit->coeff));
-	}
+	for (const auto & elem : seq)
+		deg = deg.add(ex_to<numeric>(elem.coeff));
 	return deg;
 }
 
@@ -534,10 +528,10 @@ int expairseq::compare_same_type(const basic &other) const
 	if (hashtabsize==0) {
 #endif // EXPAIRSEQ_USE_HASHTAB
 
-		epvector::const_iterator cit1 = seq.begin();
-		epvector::const_iterator cit2 = o.seq.begin();
-		epvector::const_iterator last1 = seq.end();
-		epvector::const_iterator last2 = o.seq.end();
+		auto cit1 = seq.begin();
+		auto cit2 = o.seq.begin();
+		auto last1 = seq.end();
+		auto last2 = o.seq.end();
 
 		for (; (cit1!=last1)&&(cit2!=last2); ++cit1, ++cit2) {
 			cmpval = (*cit1).compare(*cit2);
@@ -603,17 +597,18 @@ bool expairseq::is_equal_same_type(const basic &other) const
 	
 	if (hashtabsize==0) {
 #endif // EXPAIRSEQ_USE_HASHTAB
-		epvector::const_iterator cit1 = seq.begin();
-		epvector::const_iterator cit2 = o.seq.begin();
-		epvector::const_iterator last1 = seq.end();
+		auto cit1 = seq.begin();
+		auto cit2 = o.seq.begin();
+		auto last1 = seq.end();
 		
 		while (cit1!=last1) {
 			if (!(*cit1).is_equal(*cit2)) return false;
 			++cit1;
 			++cit2;
 		}
-		
-		return true;
+                
+                return true;
+
 #if EXPAIRSEQ_USE_HASHTAB
 	}
 	
@@ -647,19 +642,16 @@ unsigned expairseq::return_type() const
 	return return_types::noncommutative_composite;
 }
 
-unsigned expairseq::calchash() const
+long expairseq::calchash() const
 {
-	unsigned v = golden_ratio_hash((p_int)this->tinfo());
-	epvector::const_iterator i = seq.begin();
-	const epvector::const_iterator end = seq.end();
-	while (i != end) {
-		v ^= i->rest.gethash();
+	long v = golden_ratio_hash((p_int)this->tinfo());
+        for (const auto & elem : seq) {
+		v ^= elem.rest.gethash();
 #if !EXPAIRSEQ_USE_HASHTAB
 		// rotation spoils commutativity!
 		v = rotate_left(v);
-		v ^= i->coeff.gethash();
+		v ^= elem.coeff.gethash();
 #endif // !EXPAIRSEQ_USE_HASHTAB
-		++i;
 	}
 
 	v ^= overall_coeff.gethash();
@@ -675,9 +667,9 @@ unsigned expairseq::calchash() const
 
 ex expairseq::expand(unsigned options) const
 {
-	std::auto_ptr<epvector> vp = expandchildren(options);
+	std::unique_ptr<epvector> vp = expandchildren(options);
 	if (vp.get())
-		return thisexpairseq(vp, overall_coeff);
+		return thisexpairseq(std::move(vp), overall_coeff);
 	else {
 		// The terms have not changed, so it is safe to declare this expanded
 		return (options == 0) ? setflag(status_flags::expanded) : *this;
@@ -703,12 +695,12 @@ ex expairseq::thisexpairseq(const epvector &v, const ex &oc, bool do_index_renam
 	return expairseq(v, oc, do_index_renaming);
 }
 
-ex expairseq::thisexpairseq(std::auto_ptr<epvector> vp, const ex &oc, bool do_index_renaming) const
+ex expairseq::thisexpairseq(std::unique_ptr<epvector> vp, const ex &oc, bool do_index_renaming) const
 {
-	return expairseq(vp, oc, do_index_renaming);
+	return expairseq(std::move(vp), oc, do_index_renaming);
 }
 
-void expairseq::printpair(const print_context & c, const expair & p, unsigned upper_precedence) const
+void expairseq::printpair(const print_context & c, const expair & p, unsigned) const
 {
 	c.s << "[[";
 	p.rest.print(c, precedence());
@@ -723,8 +715,9 @@ void expairseq::printseq(const print_context & c, char delim,
 {
 	if (this_precedence <= upper_precedence)
 		c.s << "(";
-	epvector::const_iterator it, it_last = seq.end() - 1;
-	for (it=seq.begin(); it!=it_last; ++it) {
+        auto it = seq.begin();
+        auto it_last = seq.end() - 1;
+        for (; it != it_last; ++it) {
 		printpair(c, *it, this_precedence);
 		c.s << delim;
 	}
@@ -773,7 +766,7 @@ ex expairseq::recombine_pair_to_ex(const expair &p) const
 	return lst(p.rest,p.coeff);
 }
 
-bool expairseq::expair_needs_further_processing(epp it)
+bool expairseq::expair_needs_further_processing(epp)
 {
 #if EXPAIRSEQ_USE_HASHTAB
 	//#  error "FIXME: expair_needs_further_processing not yet implemented for hashtabs, sorry. A.F."
@@ -808,7 +801,7 @@ void expairseq::combine_overall_coeff(const ex &c1, const ex &c2)
 	                add_dyn(ex_to<numeric>(c1).mul(ex_to<numeric>(c2)));
 }
 
-bool expairseq::can_make_flat(const expair &p) const
+bool expairseq::can_make_flat(const expair &) const
 {
 	return true;
 }
@@ -842,7 +835,7 @@ void expairseq::construct_from_2_ex(const ex &lh, const ex &rh)
 				construct_from_2_ex_via_exvector(lh,rh);
 			} else {
 #endif // EXPAIRSEQ_USE_HASHTAB
-				if (is_a<mul>(lh) && lh.info(info_flags::has_indices) && 
+				if (is_exactly_a<mul>(lh) && lh.info(info_flags::has_indices) && 
 					rh.info(info_flags::has_indices)) {
 					ex newrh=rename_dummy_indices_uniquely(lh, rh);
 					construct_from_2_expairseq(ex_to<expairseq>(lh),
@@ -907,7 +900,7 @@ void expairseq::construct_from_2_ex(const ex &lh, const ex &rh)
 			expair p2 = split_ex_to_pair(rh);
 
 			int cmpval = p1.rest.compare(p2.rest);
-			if (cmpval==0 && unlikely(!is_a<infinity>(p1.rest))) {
+			if (cmpval==0 && unlikely(!is_exactly_a<infinity>(p1.rest))) {
 				p1.coeff = ex_to<numeric>(p1.coeff).add_dyn(ex_to<numeric>(p2.coeff));
 				if (!ex_to<numeric>(p1.coeff).is_zero()) {
 
@@ -935,10 +928,10 @@ void expairseq::construct_from_2_expairseq(const expairseq &s1,
 	combine_overall_coeff(s1.overall_coeff);
 	combine_overall_coeff(s2.overall_coeff);
 
-	epvector::const_iterator first1 = s1.seq.begin();
-	epvector::const_iterator last1 = s1.seq.end();
-	epvector::const_iterator first2 = s2.seq.begin();
-	epvector::const_iterator last2 = s2.seq.end();
+	auto first1 = s1.seq.begin();
+	auto last1 = s1.seq.end();
+	auto first2 = s2.seq.begin();
+	auto last2 = s2.seq.end();
 
 	seq.reserve(s1.seq.size()+s2.seq.size());
 
@@ -950,7 +943,7 @@ void expairseq::construct_from_2_expairseq(const expairseq &s1,
 		if (cmpval==0) {
 			// infinity evaluation is handled in the eval() method
 			// do not let infinities cancel each other here
-			if (unlikely(is_a<infinity>(first1->rest))) {
+			if (unlikely(is_exactly_a<infinity>(first1->rest))) {
 				seq.push_back(*first1);
 				seq.push_back(*first2);
 			} else {
@@ -1001,12 +994,12 @@ void expairseq::construct_from_expairseq_ex(const expairseq &s,
 		return;
 	}
 	
-	epvector::const_iterator first = s.seq.begin();
-	epvector::const_iterator last = s.seq.end();
+	auto first = s.seq.begin();
+	auto last = s.seq.end();
 	expair p = split_ex_to_pair(e);
 	// infinity evaluation is handled in eval()
 	// do not let infinities cancel each other here
-	if (unlikely(is_a<infinity>(p.rest))) {
+	if (unlikely(is_exactly_a<infinity>(p.rest))) {
 		seq.push_back(p);
 		seq.insert(seq.end(), first, last);
 		return;
@@ -1060,18 +1053,18 @@ void expairseq::construct_from_expairseq_ex(const expairseq &s,
 	}
 }
 
-void expairseq::construct_from_exvector(const exvector &v, bool hold)
+void expairseq::construct_from_exvector(const exvector &v, bool do_hold)
 {
 	// simplifications: +(a,+(b,c),d) -> +(a,b,c,d) (associativity)
 	//                  +(d,b,c,a) -> +(a,b,c,d) (canonicalization)
 	//                  +(...,x,*(x,c1),*(x,c2)) -> +(...,*(x,1+c1+c2)) (c1, c2 numeric)
 	//                  (same for (+,*) -> (*,^)
 
-	make_flat(v, hold);
+	make_flat(v, do_hold);
 #if EXPAIRSEQ_USE_HASHTAB
 	combine_same_terms();
 #else
-	if (!hold) {
+	if (!do_hold) {
 		canonicalize();
 		combine_same_terms_sorted_seq();
 	}
@@ -1096,27 +1089,23 @@ void expairseq::construct_from_epvector(const epvector &v, bool do_index_renamin
 
 /** Combine this expairseq with argument exvector.
  *  It cares for associativity as well as for special handling of numerics. */
-void expairseq::make_flat(const exvector &v, bool hold)
+void expairseq::make_flat(const exvector &v, bool do_hold)
 {
-	exvector::const_iterator cit;
-	
 	// count number of operands which are of same expairseq derived type
 	// and their cumulative number of operands
 	int nexpairseqs = 0;
 	int noperands = 0;
 	bool do_idx_rename = false;
 	
-	if (!hold) {
-		cit = v.begin();
-		while (cit!=v.end()) {
-			if (ex_to<basic>(*cit).tinfo()==this->tinfo()) {
+	if (!do_hold) {
+                for (const auto & elem : v) {
+			if (ex_to<basic>(elem).tinfo()==this->tinfo()) {
 				++nexpairseqs;
-				noperands += ex_to<expairseq>(*cit).seq.size();
+				noperands += ex_to<expairseq>(elem).seq.size();
 			}
-			if (is_a<mul>(*this) && (!do_idx_rename) &&
-					cit->info(info_flags::has_indices))
+			if (is_exactly_a<mul>(*this) && (!do_idx_rename) &&
+					elem.info(info_flags::has_indices))
 				do_idx_rename = true;
-			++cit;
 		}
 	} else
 		this->hold();
@@ -1126,26 +1115,21 @@ void expairseq::make_flat(const exvector &v, bool hold)
 	
 	// copy elements and split off numerical part
 	make_flat_inserter mf(v, do_idx_rename);
-	cit = v.begin();
-	while (cit!=v.end()) {
-		if (ex_to<basic>(*cit).tinfo()==this->tinfo() && !hold) {
-			ex newfactor = mf.handle_factor(*cit, _ex1);
+        for (const auto & elem : v) {
+		if (ex_to<basic>(elem).tinfo()==this->tinfo() && !do_hold) {
+			ex newfactor = mf.handle_factor(elem, _ex1);
 			const expairseq &subseqref = ex_to<expairseq>(newfactor);
 			combine_overall_coeff(subseqref.overall_coeff);
-			epvector::const_iterator cit_s = subseqref.seq.begin();
-			while (cit_s!=subseqref.seq.end()) {
-				seq.push_back(*cit_s);
-				++cit_s;
-			}
+                        for (const auto & elem2 : subseqref.seq)
+				seq.push_back(elem2);
 		} else {
-			if (is_exactly_a<numeric>(*cit))
-				combine_overall_coeff(*cit);
+			if (is_exactly_a<numeric>(elem))
+				combine_overall_coeff(elem);
 			else {
-				ex newfactor = mf.handle_factor(*cit, _ex1);
+				ex newfactor = mf.handle_factor(elem, _ex1);
 				seq.push_back(split_ex_to_pair(newfactor));
 			}
 		}
-		++cit;
 	}
 }
 
@@ -1153,24 +1137,20 @@ void expairseq::make_flat(const exvector &v, bool hold)
  *  It cares for associativity as well as for special handling of numerics. */
 void expairseq::make_flat(const epvector &v, bool do_index_renaming)
 {
-	epvector::const_iterator cit;
-	
 	// count number of operands which are of same expairseq derived type
 	// and their cumulative number of operands
 	int nexpairseqs = 0;
 	int noperands = 0;
 	bool really_need_rename_inds = false;
 	
-	cit = v.begin();
-	while (cit!=v.end()) {
-		if (ex_to<basic>(cit->rest).tinfo()==this->tinfo()) {
+        for (const auto & elem : v) {
+		if (ex_to<basic>(elem.rest).tinfo()==this->tinfo()) {
 			++nexpairseqs;
-			noperands += ex_to<expairseq>(cit->rest).seq.size();
+			noperands += ex_to<expairseq>(elem.rest).seq.size();
 		}
-		if ((!really_need_rename_inds) && is_a<mul>(*this) &&
-				cit->rest.info(info_flags::has_indices))
+		if ((!really_need_rename_inds) && is_exactly_a<mul>(*this) &&
+				elem.rest.info(info_flags::has_indices))
 			really_need_rename_inds = true;
-		++cit;
 	}
 	do_index_renaming = do_index_renaming && really_need_rename_inds;
 	
@@ -1179,35 +1159,31 @@ void expairseq::make_flat(const epvector &v, bool do_index_renaming)
 	make_flat_inserter mf(v, do_index_renaming);
 	
 	// copy elements and split off numerical part
-	cit = v.begin();
-	while (cit!=v.end()) {
-		if (ex_to<basic>(cit->rest).tinfo()==this->tinfo() &&
-		    this->can_make_flat(*cit)) {
-			ex newrest = mf.handle_factor(cit->rest, cit->coeff);
+        for (const auto & elem : v) {
+
+		if (ex_to<basic>(elem.rest).tinfo()==this->tinfo() &&
+		    this->can_make_flat(elem)) {
+			ex newrest = mf.handle_factor(elem.rest, elem.coeff);
 			const expairseq &subseqref = ex_to<expairseq>(newrest);
 			combine_overall_coeff(ex_to<numeric>(subseqref.overall_coeff),
-			                                    ex_to<numeric>(cit->coeff));
-			epvector::const_iterator cit_s = subseqref.seq.begin();
-			while (cit_s!=subseqref.seq.end()) {
-				seq.push_back(expair(cit_s->rest,
-				                     ex_to<numeric>(cit_s->coeff).mul_dyn(ex_to<numeric>(cit->coeff))));
-				//seq.push_back(combine_pair_with_coeff_to_pair(*cit_s,
-				//                                              (*cit).coeff));
-				++cit_s;
-			}
+			                                    ex_to<numeric>(elem.coeff));
+                        for (const auto & elem2 : subseqref.seq)
+				seq.push_back(expair(elem2.rest,
+                                        ex_to<numeric>(elem2.coeff).mul_dyn(ex_to<numeric>(elem.coeff))));
 		} else {
-			if (cit->is_canonical_numeric())
-				combine_overall_coeff(mf.handle_factor(cit->rest, _ex1));
+			if (elem.is_canonical_numeric())
+				combine_overall_coeff(mf.handle_factor(elem.rest, _ex1));
 			else {
-				ex rest = cit->rest;
-				ex newrest = mf.handle_factor(rest, cit->coeff);
-				if (are_ex_trivially_equal(newrest, rest))
-					seq.push_back(*cit);
+				const ex& rest = elem.rest;
+				const ex& newrest = mf.handle_factor(rest, elem.coeff);
+                                if (newrest.is_zero())
+                                        combine_overall_coeff(newrest);
+				else if (are_ex_trivially_equal(newrest, rest))
+					seq.push_back(elem);
 				else
-					seq.push_back(expair(newrest, cit->coeff));
+					seq.push_back(expair(newrest, elem.coeff));
 			}
 		}
-		++cit;
 	}
 }
 
@@ -1228,23 +1204,24 @@ void expairseq::combine_same_terms_sorted_seq()
 
 	bool needs_further_processing = false;
 
-	epvector::iterator itin1 = seq.begin();
-	epvector::iterator itin2 = itin1+1;
-	epvector::iterator itout = itin1;
-	epvector::iterator last = seq.end();
+	auto itin1 = seq.begin();
+	auto itin2 = itin1+1;
+	auto itout = itin1;
+	auto last = seq.end();
 	// must_copy will be set to true the first time some combination is 
 	// possible from then on the sequence has changed and must be compacted
 	bool must_copy = false;
 	while (itin2!=last) {
 		if (itin1->rest.compare(itin2->rest)==0 &&
-				unlikely(!is_a<infinity>(itin1->rest))) {
+				unlikely(!is_exactly_a<infinity>(itin1->rest))) {
 			itin1->coeff = ex_to<numeric>(itin1->coeff).
-			               add_dyn(ex_to<numeric>(itin2->coeff));
+                                add_dyn(ex_to<numeric>(itin2->coeff));
 			if (expair_needs_further_processing(itin1))
 				needs_further_processing = true;
 			must_copy = true;
 		} else {
-			if (!ex_to<numeric>(itin1->coeff).is_zero()) {
+			if (not ex_to<numeric>(itin1->coeff).is_zero()
+                                or ex_to<numeric>(itin1->coeff).is_parent_pos_char()) {
 				if (must_copy)
 					*itout = *itin1;
 				++itout;
@@ -1253,7 +1230,8 @@ void expairseq::combine_same_terms_sorted_seq()
 		}
 		++itin2;
 	}
-	if (!ex_to<numeric>(itin1->coeff).is_zero()) {
+	if (not ex_to<numeric>(itin1->coeff).is_zero()
+                or ex_to<numeric>(itin1->coeff).is_parent_pos_char()) {
 		if (must_copy)
 			*itout = *itin1;
 		++itout;
@@ -1585,8 +1563,8 @@ bool expairseq::is_canonical() const
 	if (hashtabsize > 0) return 1; // not canoncalized
 #endif // EXPAIRSEQ_USE_HASHTAB
 	
-	epvector::const_iterator it = seq.begin(), itend = seq.end();
-	epvector::const_iterator it_last = it;
+	auto it = seq.begin(), itend = seq.end();
+	auto it_last = it;
 	for (++it; it!=itend; it_last=it, ++it) {
 		if (!(it_last->is_less(*it) || it_last->is_equal(*it))) {
 			if (!is_exactly_a<numeric>(it_last->rest) ||
@@ -1618,20 +1596,20 @@ bool expairseq::is_canonical() const
  *  @see expairseq::expand()
  *  @return pointer to epvector containing expanded pairs or zero pointer,
  *  if no members were changed. */
-std::auto_ptr<epvector> expairseq::expandchildren(unsigned options) const
+std::unique_ptr<epvector> expairseq::expandchildren(unsigned options) const
 {
-	const epvector::const_iterator last = seq.end();
-	epvector::const_iterator cit = seq.begin();
+	auto cit = seq.begin();
+	auto last = seq.end();
 	while (cit!=last) {
 		const ex &expanded_ex = cit->rest.expand(options);
 		if (!are_ex_trivially_equal(cit->rest,expanded_ex)) {
 			
 			// something changed, copy seq, eval and return it
-			std::auto_ptr<epvector> s(new epvector);
+			std::unique_ptr<epvector> s(new epvector);
 			s->reserve(seq.size());
 			
 			// copy parts of seq which are known not to have changed
-			epvector::const_iterator cit2 = seq.begin();
+			auto cit2 = seq.begin();
 			while (cit2!=cit) {
 				s->push_back(*cit2);
 				++cit2;
@@ -1648,12 +1626,12 @@ std::auto_ptr<epvector> expairseq::expandchildren(unsigned options) const
 				                                           cit2->coeff));
 				++cit2;
 			}
-			return s;
+			return std::move(s);
 		}
 		++cit;
 	}
 	
-	return std::auto_ptr<epvector>(0); // signalling nothing has changed
+	return std::unique_ptr<epvector>(nullptr); // signalling nothing has changed
 }
 
 
@@ -1662,31 +1640,31 @@ std::auto_ptr<epvector> expairseq::expandchildren(unsigned options) const
  *  @see expairseq::eval()
  *  @return pointer to epvector containing evaluated pairs or zero pointer,
  *  if no members were changed. */
-std::auto_ptr<epvector> expairseq::evalchildren(int level) const
+std::unique_ptr<epvector> expairseq::evalchildren(int level) const
 {
 	// returns a NULL pointer if nothing had to be evaluated
 	// returns a pointer to a newly created epvector otherwise
 	// (which has to be deleted somewhere else)
 
 	if (level==1)
-		return std::auto_ptr<epvector>(0);
+		return std::unique_ptr<epvector>(nullptr);
 	
 	if (level == -max_recursion_level)
 		throw(std::runtime_error("max recursion level reached"));
 	
 	--level;
-	epvector::const_iterator last = seq.end();
-	epvector::const_iterator cit = seq.begin();
+	auto last = seq.end();
+	auto cit = seq.begin();
 	while (cit!=last) {
 		const ex &evaled_ex = cit->rest.eval(level);
 		if (!are_ex_trivially_equal(cit->rest,evaled_ex)) {
 			
 			// something changed, copy seq, eval and return it
-			std::auto_ptr<epvector> s(new epvector);
+			std::unique_ptr<epvector> s(new epvector);
 			s->reserve(seq.size());
 			
 			// copy parts of seq which are known not to have changed
-			epvector::const_iterator cit2=seq.begin();
+			auto cit2=seq.begin();
 			while (cit2!=cit) {
 				s->push_back(*cit2);
 				++cit2;
@@ -1703,12 +1681,12 @@ std::auto_ptr<epvector> expairseq::evalchildren(int level) const
 				                                           cit2->coeff));
 				++cit2;
 			}
-			return s;
+			return std::move(s);
 		}
 		++cit;
 	}
 	
-	return std::auto_ptr<epvector>(0); // signalling nothing has changed
+	return std::unique_ptr<epvector>(nullptr); // signalling nothing has changed
 }
 
 /** Member-wise substitute in this sequence.
@@ -1716,7 +1694,7 @@ std::auto_ptr<epvector> expairseq::evalchildren(int level) const
  *  @see expairseq::subs()
  *  @return pointer to epvector containing pairs after application of subs,
  *    or NULL pointer if no members were changed. */
-std::auto_ptr<epvector> expairseq::subschildren(const exmap & m, unsigned options) const
+std::unique_ptr<epvector> expairseq::subschildren(const exmap & m, unsigned options) const
 {
 	// When any of the objects to be substituted is a product or power
 	// we have to recombine the pairs because the numeric coefficients may
@@ -1724,8 +1702,8 @@ std::auto_ptr<epvector> expairseq::subschildren(const exmap & m, unsigned option
 	if (!(options & (subs_options::pattern_is_product | subs_options::pattern_is_not_product))) {
 
 		// Search the list of substitutions and cache our findings
-		for (exmap::const_iterator it = m.begin(); it != m.end(); ++it) {
-			if (is_exactly_a<mul>(it->first) || is_exactly_a<power>(it->first)) {
+		for (const auto & elem : m) {
+			if (is_exactly_a<mul>(elem.first) || is_exactly_a<power>(elem.first)) {
 				options |= subs_options::pattern_is_product;
 				break;
 			}
@@ -1737,7 +1715,7 @@ std::auto_ptr<epvector> expairseq::subschildren(const exmap & m, unsigned option
 	if (options & subs_options::pattern_is_product) {
 
 		// Substitute in the recombined pairs
-		epvector::const_iterator cit = seq.begin(), last = seq.end();
+		auto cit = seq.begin(), last = seq.end();
 		while (cit != last) {
 
 			const ex &orig_ex = recombine_pair_to_ex(*cit);
@@ -1745,7 +1723,7 @@ std::auto_ptr<epvector> expairseq::subschildren(const exmap & m, unsigned option
 			if (!are_ex_trivially_equal(orig_ex, subsed_ex)) {
 
 				// Something changed, copy seq, subs and return it
-				std::auto_ptr<epvector> s(new epvector);
+				std::unique_ptr<epvector> s(new epvector);
 				s->reserve(seq.size());
 
 				// Copy parts of seq which are known not to have changed
@@ -1760,7 +1738,7 @@ std::auto_ptr<epvector> expairseq::subschildren(const exmap & m, unsigned option
 					s->push_back(split_ex_to_pair(recombine_pair_to_ex(*cit).subs(m, options)));
 					++cit;
 				}
-				return s;
+				return std::move(s);
 			}
 
 			++cit;
@@ -1769,14 +1747,14 @@ std::auto_ptr<epvector> expairseq::subschildren(const exmap & m, unsigned option
 	} else {
 
 		// Substitute only in the "rest" part of the pairs
-		epvector::const_iterator cit = seq.begin(), last = seq.end();
+		auto cit = seq.begin(), last = seq.end();
 		while (cit != last) {
 
 			const ex &subsed_ex = cit->rest.subs(m, options);
 			if (!are_ex_trivially_equal(cit->rest, subsed_ex)) {
 			
 				// Something changed, copy seq, subs and return it
-				std::auto_ptr<epvector> s(new epvector);
+				std::unique_ptr<epvector> s(new epvector);
 				s->reserve(seq.size());
 
 				// Copy parts of seq which are known not to have changed
@@ -1791,7 +1769,7 @@ std::auto_ptr<epvector> expairseq::subschildren(const exmap & m, unsigned option
 					s->push_back(combine_ex_with_coeff_to_pair(cit->rest.subs(m, options), cit->coeff));
 					++cit;
 				}
-				return s;
+				return std::move(s);
 			}
 
 			++cit;
@@ -1799,7 +1777,7 @@ std::auto_ptr<epvector> expairseq::subschildren(const exmap & m, unsigned option
 	}
 	
 	// Nothing has changed
-	return std::auto_ptr<epvector>(0);
+	return std::unique_ptr<epvector>(nullptr);
 }
 
 
