@@ -99,7 +99,13 @@ static ex real_part_evalf(const ex & arg, PyObject* parent)
 
 static ex real_part_eval(const ex & arg)
 {
-	return arg.real_part();
+        ex pat = pow(wild(1), wild(2));
+        lst l;
+        if (arg.find(pat, l))
+                // real parts of powers can be expensive
+                return arg.expand().real_part();
+        else
+                return arg.real_part();
 }
 
 static void real_part_print_latex(const ex & arg, const print_context & c)
@@ -144,7 +150,13 @@ static ex imag_part_evalf(const ex & arg, PyObject* parent)
 
 static ex imag_part_eval(const ex & arg)
 {
-	return arg.imag_part();
+        ex pat = pow(wild(1), wild(2));
+        lst l;
+        if (arg.find(pat, l))
+                // imag parts of powers can be expensive
+                return arg.expand().imag_part();
+        else
+                return arg.imag_part();
 }
 
 static void imag_part_print_latex(const ex & arg, const print_context & c)
@@ -192,14 +204,51 @@ static ex abs_eval(const ex & arg)
 	if (is_exactly_a<numeric>(arg))
 		return abs(ex_to<numeric>(arg));
 
-	if (arg.info(info_flags::nonnegative))
+	if (arg.info(info_flags::nonnegative) or arg.info(info_flags::positive))
 		return arg;
 
-	if (is_ex_the_function(arg, abs))
-		return arg;
+	if (arg.info(info_flags::negative) or (-arg).info(info_flags::nonnegative))
+		return -arg;
 
-	if (is_ex_the_function(arg, exp))
-		return exp(arg.op(0).real_part());
+        if (is_exactly_a<function>(arg)) {                
+                if (is_ex_the_function(arg, abs))
+                        return arg;
+                if (is_ex_the_function(arg, exp))
+                        return exp(arg.op(0).real_part());
+                if (is_ex_the_function(arg, conjugate_function))
+                        return abs(arg.op(0));
+                if (is_ex_the_function(arg, step))
+                        return arg;
+                return abs(arg).hold();
+        }
+
+        if (is_exactly_a<mul>(arg)) {
+                ex prod = _ex1;
+                ex prod_sy = _ex1;
+                bool is_prod_neg = false;
+                for (size_t i=0; i<arg.nops(); ++i) {
+                        const ex& factor = arg.op(i);
+                        if (has_symbol(factor))
+                                prod_sy *= factor;
+                        else if (factor.info(info_flags::real)) {
+                                if (factor.info(info_flags::negative)) {                                        
+                                        is_prod_neg = not is_prod_neg;
+                                        prod *= factor;
+                                }
+                                else if (factor.info(info_flags::positive))                                        
+                                        prod *= factor;
+                                else
+                                        prod *= abs(factor).hold();
+                        }
+                        else
+                                prod *= abs(factor);
+                }
+                if (is_prod_neg)
+                        prod *= _ex_1;
+                if (not prod_sy.is_integer_one())
+                        prod *= abs(prod_sy).hold();
+                return prod;
+	}
 
 	if (is_exactly_a<power>(arg)) {
 		const ex& base = arg.op(0);
@@ -208,12 +257,6 @@ static ex abs_eval(const ex & arg)
 			return pow(abs(base), exponent.real_part());
 	}
 
-	if (is_ex_the_function(arg, conjugate_function))
-		return abs(arg.op(0));
-
-	if (is_ex_the_function(arg, step))
-		return arg;
-
 	return abs(arg).hold();
 }
 
@@ -244,13 +287,12 @@ static ex abs_imag_part(const ex& arg)
 
 static ex abs_power(const ex & arg, const ex & exp)
 {
-	if ((is_a<numeric>(exp) && ex_to<numeric>(exp).is_even()) || exp.info(info_flags::even)) {
-		if (arg.info(info_flags::real) || arg.is_equal(arg.conjugate()))
-			return power(arg, exp);
-		else
-			return power(arg, exp/2)*power(arg.conjugate(), exp/2);
-	} else
-		return power(abs(arg), exp).hold();
+	if (((is_exactly_a<numeric>(exp) and ex_to<numeric>(exp).is_even()) 
+                or exp.info(info_flags::even))
+                and (arg.info(info_flags::real) or arg.is_equal(arg.conjugate())))
+	        return power(arg, exp);
+	else
+	        return power(abs(arg), exp).hold();
 }
 
 static ex abs_deriv(const ex & x, unsigned deriv_param)
