@@ -222,7 +222,7 @@ int print_order::compare(const basic &lh, const basic &rh) const
 	else if (typeid_rh == function_id())
 		//print functions before fderivatives, after anything else
 		return 1;
-	else if (typeid_lh == mul_id())
+	else if (typeid_lh == mul_id()) {
 		if (typeid_rh == power_id())
 			return compare_mul_power(
 					static_cast<const mul&>(lh),
@@ -236,7 +236,8 @@ int print_order::compare(const basic &lh, const basic &rh) const
 					static_cast<const add&>(rh),
 					static_cast<const mul&>(lh));
 		else return generic_compare(typeid_lh, typeid_rh);
-	else if (typeid_lh == add_id())
+                }
+        else if (typeid_lh == add_id()) {
 		if (typeid_rh == power_id())
 			return compare_add_power(
 					static_cast<const add&>(lh),
@@ -250,7 +251,8 @@ int print_order::compare(const basic &lh, const basic &rh) const
 					static_cast<const add&>(lh),
 					static_cast<const mul&>(rh));
 		else return generic_compare(typeid_lh, typeid_rh);
-	else if (typeid_lh == power_id())
+                }
+	else if (typeid_lh == power_id()) {
 		if (typeid_rh == mul_id())
 			return -compare_mul_power(
 					static_cast<const mul&>(rh),
@@ -264,7 +266,8 @@ int print_order::compare(const basic &lh, const basic &rh) const
 					static_cast<const power&>(lh),
 					static_cast<const symbol&>(rh));
 		else return generic_compare(typeid_lh, typeid_rh);
-	else if (typeid_lh == symbol_id())
+                }
+	else if (typeid_lh == symbol_id()) {
 		if (typeid_rh == mul_id())
 			return -compare_mul_symbol(
 					static_cast<const mul&>(rh),
@@ -278,6 +281,7 @@ int print_order::compare(const basic &lh, const basic &rh) const
 					static_cast<const power&>(rh),
 					static_cast<const symbol&>(lh));
 		else return generic_compare(typeid_lh, typeid_rh);
+                }
         return generic_compare(typeid_lh, typeid_rh);
 }
 
@@ -328,7 +332,7 @@ int print_order::compare_mul_power(const mul &lh, const power &rh) const
 	double lh_deg = lh.total_degree();
 	double rh_deg = 1;
 	numeric rh_exp;
-	if (is_a<numeric>(rh.exponent)) {
+	if (is_exactly_a<numeric>(rh.exponent)) {
 		rh_deg = numeric_to_double(ex_to<numeric>(rh.exponent));
 	}
 	if (rh_deg != lh_deg)
@@ -376,10 +380,10 @@ int print_order::compare_same_type_mul(const mul &lh, const mul &rh) const
 	// compare each item in lh to corresponding element in rh
 	const epvector & sorted_seq1 = lh.get_sorted_seq();
 	const epvector & sorted_seq2 = rh.get_sorted_seq();
-	epvector::const_reverse_iterator cit1 = sorted_seq1.rbegin();
-	epvector::const_reverse_iterator cit2 = sorted_seq2.rbegin();
-	epvector::const_reverse_iterator last1 = sorted_seq1.rend();
-	epvector::const_reverse_iterator last2 = sorted_seq2.rend();
+	auto cit1 = sorted_seq1.rbegin();
+	auto cit2 = sorted_seq2.rbegin();
+	auto last1 = sorted_seq1.rend();
+	auto last2 = sorted_seq2.rend();
 
 	for (; (cit1!=last1)&&(cit2!=last2); ++cit1, ++cit2) {
 		// compare bases
@@ -502,10 +506,10 @@ int print_order::compare_same_type_add(const add &lh, const add &rh) const
 
 	const epvector & sorted_seq1 = lh.get_sorted_seq();
 	const epvector & sorted_seq2 = rh.get_sorted_seq();
-	epvector::const_iterator cit1 = sorted_seq1.begin();
-	epvector::const_iterator cit2 = sorted_seq2.begin();
-	epvector::const_iterator last1 = sorted_seq1.end();
-	epvector::const_iterator last2 = sorted_seq2.end();
+	auto cit1 = sorted_seq1.begin();
+	auto cit2 = sorted_seq2.begin();
+	auto last1 = sorted_seq1.end();
+	auto last2 = sorted_seq2.end();
 
 	for (; (cit1!=last1)&&(cit2!=last2); ++cit1, ++cit2) {
 		// compare bases
@@ -560,7 +564,7 @@ int print_order::compare_power_symbol(const power &lh,
 	int cmpval;
 
 	// We are in an add object
-	if (is_a<numeric>(lh.exponent)) {
+	if (is_exactly_a<numeric>(lh.exponent)) {
 		double lh_deg = numeric_to_double(ex_to<numeric>(lh.exponent));
 		if (lh_deg != 1)
 			return lh_deg < 1 ? -1 : 1;
@@ -597,10 +601,10 @@ int print_order::compare_same_type_power(const power &lh, const power &rh) const
 
 	double lh_deg = 1;
 	double rh_deg = 1;
-	if (is_a<numeric>(lh.exponent)) {
+	if (is_exactly_a<numeric>(lh.exponent)) {
 		lh_deg = numeric_to_double(ex_to<numeric>(lh.exponent));
 	}
-	if (is_a<numeric>(rh.exponent)) {
+	if (is_exactly_a<numeric>(rh.exponent)) {
 		rh_deg = numeric_to_double(ex_to<numeric>(rh.exponent));
 	}
 	if (rh_deg != lh_deg)
@@ -610,7 +614,7 @@ int print_order::compare_same_type_power(const power &lh, const power &rh) const
 	if (cmpval != 0)
 		return cmpval;
 
-	if (is_a<numeric>(lh.exponent) && is_a<numeric>(rh.exponent))
+	if (is_exactly_a<numeric>(lh.exponent) && is_exactly_a<numeric>(rh.exponent))
 		return 0;
 	return compare(lh.exponent, rh.exponent);
 }
@@ -632,7 +636,7 @@ template <template <class T, class = std::allocator<T> > class C>
 int print_order::compare_same_type_container(const container<C> &lh,
 					     const container<C> &rh) const
 {
-	typename C<ex>::const_iterator it1 = lh.seq.begin(), it1end = lh.seq.end(),
+	auto it1 = lh.seq.begin(), it1end = lh.seq.end(),
 			      it2 = rh.seq.begin(), it2end = rh.seq.end();
 
 	while (it1 != it1end && it2 != it2end) {
